<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="kturtle" version="5">

<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_new" />
    <Separator />
    <Action name="file_open" />
    <Action name="file_recent" />
    <Menu name="examples"><text>&amp;Examples</text>
      <ActionList name="examples_actionlist" />
    </Menu>
    <Action name="get_new_examples" />
    <Separator />
    <Action name="file_save" />
    <Action name="file_save_as" />
    <Separator />
    <Action name="file_export_to_html" />
    <Action name="file_print" />
    <Separator />
    <Action name="file_quit" />
  </Menu>
  <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
    <Action name="edit_undo" />
    <Action name="edit_redo" />
    <Separator />
    <Action name="edit_cut" />
    <Action name="edit_copy" />
    <Action name="edit_paste" />
    <Separator />
    <Action name="edit_select_all" />
    <Separator />
    <Action name="edit_find" />
    <Action name="edit_find_next" />
    <Action name="edit_find_prev" />
    <Action name="edit_replace" />
    <Separator />
    <Action name="overwrite" />
  </Menu>
  <Menu name="canvas"><text>&amp;Canvas</text>
    <Action name="canvas_export_to_png" />
    <Action name="canvas_export_to_svg" />
    <Separator />
    <Action name="canvas_print" />
  </Menu>
  <Menu name="run"><text>&amp;Run</text>
    <Action name="run" />
    <Action name="pause" />
    <Action name="abort" />
    <Menu name="speed"><text>&amp;Run Speed</text>
      <Action name="dedicated_speed" />
      <Action name="full_speed" />
      <Action name="slow_speed" />
      <Action name="slower_speed" />
      <Action name="slowest_speed" />
      <Action name="step_speed" />
    </Menu>
  </Menu>
  <Menu name="tools"><text>&amp;Tools</text>
    <Action name="direction_chooser" />
    <Action name="color_picker" />
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Menu name="language"><text>&amp;Script Language</text>
      <ActionList name="languages_actionlist" />
    </Menu>
    <Separator />
    <Action name="show_editor" />
    <Action name="show_inspector" />
    <Action name="show_errors" />
    <Separator />
    <Action name="line_numbers" />
  </Menu>
  <Menu name="help"><text>&amp;Help</text>
    <Action name="context_help" />
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" fullWidth="true" newline="true" noMerge="1"><text>Main Toolbar</text>
  <Action name="file_new" />
  <Action name="file_open" />
  <Separator />
  <Action name="file_save" />
  <Action name="file_save_as" />
  <Separator />
  <Action name="run_speed" />
  <Action name="pause" />
  <Action name="abort" />
  <Separator />
  <Action name="console" />
  <Action name="execute_console" />
</ToolBar>

</gui>
